import { baseUrl } from '@/config' 
import  Toast from "../toast/index.js"
import { isObeject, isFunction } from "@/utils/types.js"
import Validator from "@/utils/validate.js"
import { login_slilen } from '@/utils/modules/login.js'
import store from '@/store/index.js'

/**
 * @desc 请求 所有的状态码都在这里处理
 * @param { Object } options 参考uni.request参数说明
 * @return {
	status: true / false
	msg
	data
 }
 */
function validateOps(options = {}) {
	const validator = new Validator()
	validator.add(Object.keys(options), [{strategy: 'minLength:1', errorMsg: '参数格式不正确'}])
	validator.add(options.url, [{strategy: 'notEmpty', errorMsg: '请求地址不能为空'}])
	return validator.validate()
}

// 没有考虑到上传，先前没有统一响应处理
async function successRes(res, options, resolve, reject) {
	try{
		switch(res.statusCode) {
		    case 200: 
		        options.successCb && typeof options.successCb === 'function' && options.successCb()
		        // 处理某些接口返回String类型的数据
		        const data = (res.data && typeof res.data === 'string') ? JSON.parse(res.data) : res.data
				if(!data) return Toast({title: '后端没有返回data'})
		        const status = data.code
				switch(status) {
					case 10:
						if(store.state.user.isOverdue) {
							store.state.user.isOverdue = false
						}
						resolve({
						    status: true,
						    data: data.data,
							test: data,
						    msg: data.rep_msg
						})
						break;
					case -1:
						Toast({title: data.rep_msg})
						reject({
						    status: false,
						    data: data.data,
						    msg: data.rep_msg
						})
						break;
					case -2:
						return Toast({title: data.rep_msg})
						break;
					case -3:
						Toast({
							title: '登录已失效，请重新登录',
							cb: () => uni.navigateTo({
								url: '/pages/login/index'
							})
						})
						break;
					case -11: // 购买的服务过期
						if(!store.state.user.isOverdue) {
							store.state.user.isOverdue = true
						}
						return Toast({title: data.rep_msg})
						break;
					default:
						Toast({title: data.rep_msg})
						reject({
							status: false,
							code: status, // 正常不返回为了好找 后端调试 -1 => 业务错误（一般失败情况返回此项）;-2 => 系统异常; -3 => 请先登录; -4 => 请求失败; -10 => 签名失败
							data: data.data,
							msg: data.rep_msg
						})
						break;
				}
		        break;
		    case 404:
		        return Toast({title: '请求路径不存在'})                        
		        break;
		    case 500:
		        return Toast({title: '服务器错误'})
		        break; // 这里的break可以不写，不会执行到。return直接退出函数了
		    default:
		        Toast({title: '网络请求失败'})
		        return
		        break;    
		}
	}catch(e){
		console.log(e)
		Toast({title: e.msg  || '程序请求错误'})
	}
}

// 请求
export function request (options) {
    const valite_err = validateOps(options)
	if(valite_err) return Toast({title: valite_err})
	options.method = options.method ? options.method.toUpperCase() : "GET" // 默认get
	options.url = `${baseUrl}/${options.url}`
	if(options.needMask) {
		uni.showLoading({title: '加载中...', mask: true})
	}
    return new Promise((resolve, reject) => {
		// const isOverdue = store.state.user.isOverdue
		// if(isOverdue) {
		// 	reject({
		// 		status: false,
		// 		data: null,
		// 		msg: '服务已过期'
		// 	})
		// 	return
		// }
		const token = store.state.user.token || ''
		const appid = store.state.user.appid || ''
        uni.request({
            ...options,
            header: {
				'content-type': 'application/x-www-form-urlencoded',
				'token': token,
				'appid': appid
			},
            success: res => {
                successRes(res, options, resolve, reject)
            },
            fail: err => {
				Toast({title: '网络开小差了,请检查网络!'})
                options.errCb && typeof options.errCb === 'function' && options.errCb();
                reject({
                    status: false,
                    code: 400,
                    data: err
                })
            },
            complete: () => {
                options.completeCb && typeof options.completeCb === 'function' && options.completeCb();
				options.needMask && uni.hideLoading()
            }
        })
    })
}

// 上传文件
export function upload(options) {
	const valite_err = validateOps(options)
	if(valite_err) return Toast({title: valite_err})
	options.url = `${baseUrl}/${options.url}`
	uni.showLoading({title: '上传中...', mask: true})
	return new Promise((resolve, reject) => {
		// const isOverdue = store.state.user.isOverdue
		// if(isOverdue) {
		// 	reject({
		// 		status: false,
		// 		data: null,
		// 		msg: '服务已过期'
		// 	})
		// 	return
		// }
		const token = store.state.user.token || ''
		const appid = store.state.user.appid || ''
		uni.uploadFile({
			url: options.url, 
			filePath: options.tempFile,
			header: {
				// "Content-Type": "multipart/form-data",
				'token': token,
				'appid': appid
			},
			name: 'file',
			success: res =>{
				successRes(res, options, resolve, reject)
			},
			fail: err => {
				Toast({title: '网络开小差了,请检查网络!'})
				options.errCb && typeof options.errCb === 'function' && options.errCb();
				reject({
				    status: false,
				    code: 400,
				    data: err
				})
			},
			complete: () => {
			    options.completeCb && typeof options.completeCb === 'function' && options.completeCb();
				uni.hideLoading()
			}
		})
	})
}
